//*****************************************************************************
// Filename: sortTable.js
// Description: This javascript file can be applied to convert record tables
// in a HTML file to be client-side sortable by associating title columns with
// sort events. 
//
// COPYRIGHT (C) 2002 HAN J. YU
// THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT 
// UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE 
// SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) 
// ANY LATER VERSION. THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE 
// USEFUL, BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF 
// MERCHANTABILITY OF FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU GENERAL 
// PUBLIC LICENSE FOR MORE DETAILS. 
//
// YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE ALONG 
// WITH THIS PROGRAM; IF NOT, WRITE TO: 
//
// THE FREE SOFTWARE FOUNDATION, INC., 
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA 
//
// Bugs/Comments: han@velocityhsi.com
//
// Change History:
//
// 11-26-01: 
//	o Made a few more settings configurable (i.e. data delimiter etc).
//	o Added a check for the browser type/version (>= IE 5.0 allowed).
// 11-27-01:
//	o Used document.getElementById method to retrieve object
//	o Now supports both IE 5.0 or greater and Netscape 6.0 or greater
// 11-28-01:
//	o Fixed the status display for Netscape.
//	o Fixed the cursor shape for Netscape. Used pointer instead of hand.
// 11-29-01:
//	o Fixed a cursor bug for IE 5.5
// 12-03-01:
//	o Uses delete/insert cell when doing the sort.
//	o Uses delete/insert cell for the title.
// 12-04-01:
//	o Created redrawTitle function to tidy things up a bit.
// 12-09-01:
//	o Used the innerMost cell's nodeValue to display title name.
// 12-10-01:
//	o Now preserves the align property of the title cell.
//	o Allows to have multiple child nodes inside the title cell.
//	o Simulates <TH> by centeing and bold-facing title cell contents.
// 01-21-02: (Thanks to Ric Shumack ...)
//	o Added one more condition for table init call (table.tainted == false)
//	o Used the preferred method for checking the browser version
// 03-01-02: (Recommendation from Dan ...)
//	o Default sorting will be applied to all tables upon loading
// 03-12-02:
//	o Changed parseInt to parseFloat in compare function
// 03-13-02:
//	o Ignores THEAD, TFOOT in the data row list
// 03-18-02:
//	o Compare as numbers only if both inputs are number
// 03-19-02:
//	o Now compares IETF recognized date string inputs
// 03-29-02: (Recommendation from Ashley ...)
//	o New global settings for up/down images and title font face
// 03-31-02:
//	o Retains various cell properties
//	o align, vAlign, bgColor, noWrap, width, height
//	o borderColor, borderColorLight, borderColorDark
//	o Bug fix in isDate method.
// 04-04-02: (Thanks to Steen ...)
//	o Can have separate table for titles
// 04-06-02: (Thanks to Steen ...)
//	o CSS properties are preserved
//	o Bug fix (&nbsp; removed)
//	o Title cell events applied to cells
// 04-08-02:
//	o Used eval to simplify the code
// 04-24-02:
//	o Removes HTML tags when comparing cell content
// 05-07-02:
//	o Handles Euro date format
// 05-15-02:
//	o Fixed up/down replacement reg exp
// 06-13-02:
//	o Title column hints (via TITLE) added
// 07-12-02:
//	o Changed up/down char to up/down arrows
//	o Up/down arrows get displayed if doSortUponLoad is set 
// 07-15-02:
//	o Added a new feature to allow skipping columns when sorting
//	o This requires 2nd parameter to specify which columns to skip
// 07/19-02:
//	o Title cells can be highlighted (bgcolor chaged) onMouseOver ... 
// 09-04-02:
//	o Bug fix in cell highlighting functionality 
// 09-06-02:
//	o Mouse pointer becomes hourglass while sorting
// 09-21-02:
//	o Sorting is mistakenly done twice if doSortUponLoad is set
//	o Mouse pointer to hourglass while sorting ... (bug fixes)
//	o Use "" as default in cursor settings
//	o DOES NOT WORK WITH NETSCAPE ... TO BE FIXED SOON !!!
// 09-22-02:
//	o Made it work with Netscape
// 09-23-02:
//	o Fixed error if doSortUponLoad is NOT set
//	o Sort direction alternates even between tables 
//*****************************************************************************

//*****************************************************************************
// sortTable.js
//
// This script contains useful functions that can be used to convert ordinary
// tables into sortable tables by modifying the HTML sources.
// 
// Here is how one can do that. The following assumptions are required
// for the tables to be sorted.
//
// 1. All the record columns must be the same lengh. Otherwise (i.e. the ones
//    that contain colspan) the rows will be ignored. 
//
// 2. Row spans can not happen in the record rows though column spans
//    can be one of the record rows.
//
// 3. Row-spanned single column will be considered as title.
//
// To enable the sorting, simply include this javascript source file and
// add an onLoad event to the <body> like below:
//
// <body onLoad='initTable("table1");initTable("table2");' ...>
//
// Note that all the tables that need to be sorted MUST contain ID tag. 
// So, if they do not exist, you must create one for each table that
// needs to be sorted. Also, the table names/ids MUST BE UNIQUE.
//*****************************************************************************

/**  
*  SAMPLE CODE TO MAKE A TABLE WITHIN A NVIP SERVLET SORTABLE
*
*  1.  give the table an id name
*  2.  modify <BODY> tag:				    
*
*  <BODY onLoad='initTable(\"stackview_table\", \"0\", \"\u30e2\u30cb\u30bf\u30fc\u4e2d\");'>
*     -->   The second argument is optional; It is used to indicate the column indices that are
*            non-sortable
*     -->   The third argument supplies the string that is to be used to display a message
*            in the browser.  In English, this String is "Click to sort by"  For internationalization, 
*            this string should probably come from a resource bundle.   
*                  
*				 				    
*  3.  add the following <Script> tag within the <Body> tags                            
*  
*  "<SCRIPT SRC=\"" + company + "/javascript/sortTable.js\"></SCRIPT>"
*
**/

// Global variables
var table;				// Table object
var ttable;				// Title Table object
var tableId;				// Current Table ID
var ttableId;				// Current Title Table ID
var tableIdArray = new Array();		// Holds Tabld IDs
var sortDirArray = new Array();		// Holds sort direction (descending)  
var tableCount = 0;			// Total table count
var rowArray = new Array();		// Data row array
var rowIdArray = new Array();		// Data row ID array
var rowPropArray = new Array();		// Data row property array
var titleRowArray = new Array();	// Contains row pointer for titles
var titleInnerHTMLArray = new Array();	// Contains innerHTML for title cells
var titleSpanCountArray = new Array();	// Contains the row-span count
var titleRowCellArray = new Array();	// Dynamically constructed title cells
var titleSpanCellArray = new Array();	// Title elelments from row-spanned
var colSpanArray = new Array();		// Rows col-spanned
var colTitleFilled = new Array();	// Indicates whether title is filled
var sortIndex;				// Selected index for sort
var nRow, actualNRow, maxNCol;		// Various table stats
var origColor;				// Holds original default color
var isIE;				// True if IE
var separateTitle = false;		// Has separate title table
var linkEventString =			// What's insider <a> tag
	'onMouseOver=\'setCursor(this);' +
	'setColor(this,"selected");\' ' +
	'onMouseOut=\'setColor(this,"default");\' ' +
	'onClick="setCursorWait(this);setTimeout(\'sortTable(';

var cellPropArray = new Array(		// Cell properties to preserve
	"align", "vAlign", "bgColor");

var colsToIgnoreArray;			// Col indexes to ignore (Array)
var colsToIgnore;			// Col indexes to ignore

//ADDED BY HA N.
//************************************************************************
//These 2 variables were added by Ha N. in order to internationalize the window status.
//The messages are in English by default.
var statusString = "Click to sort by";
var sortProgressString = "% of sorting completed...";

//this var added by Ha N. in order to sort internationalized dates
var datePattern = null;
//this var added by Ha N. in order to sort internationalized times
var shortTimePattern = null;
var mediumTimePattern = null;
var longTimePattern = null;
//this var indicates if this is the very first initialization of the table
var firstTableInit = true;
var doRowCheck = false;
var doInit_global = false;
//************************************************************************


// Configurable constants
var useImg = true;		// Uses images if true
//=====================================================================
//the following 2 images were downloaded (and modified) by Ha N. from 
//http://www.fresherimage.com/Icons/arrow/
//=====================================================================
var ascChrFile = "/nvip/images/upArrow.gif";	// Image for ascending sort
var desChrFile = "/nvip/images/downArrow.gif";	// Image for descending sort
var ascChr = "&uarr;";		// Symbol for ascending sort
var desChr = "&darr;";		// Symbol for descending sort
var useCustomTitleFont = false;	// Uses custom fonts for titles
var titleFont = "Trebuchet MS";	// Title Font
var selectedColor = "blue";	// Color for sort focus
var defaultColor = "black";	// Default color for sort off-focus
var recDelimiter = '|';		// Char used as a record separator
var cellPropDelimiter = ",";	// Char used as a cell prop separator
var updownColor = 'gray';	// Specified the color for up/downs

//changed by Ha N. from true to false in order to prevent slowing of servlet pages
var doSortUponLoad = false;	// Do the sort upon page load
var defaultSortColumn = 0;	// Indicates the default sort column index
//var useEuroDate = false;	// Set if Euro Date format is used
var highlightSelCell = false;	// Set if selected cell needs to be highlighted
var cellHighlightColor = "red";	// Color for cell highlight

//*****************************************************************************
// Main function. This is to be associated with onLoad event in <BODY>. 
//
// IMPORTANT: This is the only function that needs to be included in the pages
// to be sorted. The rest of the functions are simply called by this
// function.
//
// Ha N: The argument statusStringArg was added in order to internationalize the
// window status message. datePatternArg was added in order to sort internationalized
// dates.  If the table does not contains dates, then this arg should be null.  If
// there are dates to compare, the servlet containing the table should set this 
// argument to the value of 'Localizer.getShortDatePattern(locale, false);'
//
//REMINDER TO SELF:  IN ADDING NEW ARGS, DON'T FORGET TO MODIFY THE OTHER
//2 CALLS TO INITTABLE!
//*****************************************************************************
var disableSort = true;



function initTable(obj, statusStringArg, sortProgressStringArg, datePatternArg, timePatterns, ignore, rowCheck)
{

   if(!disableSort)
   {
   

   //alert("num of items in doc:  " + document.all.length);
	var tIndex;

	// Check whether it's viewed by IE 5.0 or greater
	if (! checkBrowser()) return;

   //=====================================
   //Process function arguments
   //=====================================
   doRowCheck = rowCheck;
	colsToIgnore = ignore;
	if (colsToIgnore != null) 
		colsToIgnoreArray = ignore.split(",");
	else
		colsToIgnore = null;

   //added by Ha N.
   if(statusStringArg != null) 
		statusString = statusStringArg;

   if(sortProgressStringArg != null) 
		sortProgressString = sortProgressStringArg;

   if(datePatternArg != null)
      datePattern = datePatternArg;

   if(timePatterns != null)
   {
      var patternArry = timePatterns.split("*");
      shortTimePattern = patternArry[0]; 
      mediumTimePattern = patternArry[1];
      longTimePattern = patternArry[2];
   }
   //=====================================
   
	// Local variables
	var countCol;
	var currentCell;
	var nColSpan, nRowSpannedTitleCol, colPos;
	var titleFound = false;
	var skipRow = false;
	var rNRowSpan, rNColSpan, parentNodeName;
	var cCellContent, cCellSetting;
	var sObj;
	var ctable;
	var doneftable = false;
	var cmd;
	var tableLoaded = false;

   //============================MODIFIED BY HA N===============================
   //If the table has already been initialized once, don't do this again.
   //This modification is necessary because some NVIP tables contain rows
   //that act purely as dividers.  Each data cell of these divider rows contain
   //the "*" symbol.  Upon the first call to sortTable(), these rows are
   //removed, the table is reinitialized to reflect the deletion of these rows,
   //and then the table is sorted.  
   //===========================================================================
   if(firstTableInit)
   {
   	// Initializing global table object variable
   	if (obj.tagName == "TABLE")
   	{
   		// Assumes that the obj is THE OBJECT
   		table = obj;
   	}
   	else
   	{
   		table = document.getElementById(obj);
   		sObj = obj.id + "title";
   		ttable = document.getElementById(sObj);
   	}
       
   	// Check whether it's an object
   	if (table == null) return;
   
   	// Check whether it's a table
   	if (table.tagName != "TABLE") return;
   
   	if (ttable != null && ttable.tagName == "TABLE")
   		separateTitle = true;
   
   	// No need to re-init if it's already done
   	if (tableId == table.id && table.tainted == false) return;
   
   	// Setting table id
   	tableId = table.id;

      if (separateTitle) ttableId = ttable.id;
   }//end of if(firstTableInit)

   //============================MODIFIED BY HA N==================
   //The following while loop added to resolve a bug that prevented
   //tables with rows containing data cells with colSpan > 1 
   //(i.e. Syspoint) from being sorted correctly.
   //The original code was:   
   //maxNCol = table.rows[table.rows.length-1].cells.length;
   //===============================================================
   maxNCol = 0;
   var numOfDataCellsInRow = 0;
   var i = 0;
   var numOfRows = table.rows.length;
   while(i < numOfRows)
   {
      numOfDataCellsInRow = table.rows[i].cells.length;
      if (numOfDataCellsInRow > maxNCol) 
      {
         maxNCol = numOfDataCellsInRow;
      }
      i++; 
   }

	// Initializing arrays
	rowArray = new Array();
	rowIdArray = new Array();
	rowPropArray = new Array();
	colSpanArray = new Array();
	colTitleFilled = new Array();
	titleRowArray = new Array();
	titleInnerHTMLArray = new Array();
	titleSpanCountArray = new Array();
	titleRowCellArray = new Array();
	
	for (var i=0; i<maxNCol; i++)
	{
		if (i == 0) continue;
		colTitleFilled[i] = false;
	}

	// Setting the number of rows
	nRow = table.rows.length;	

   /**
   *  Modified by Ha N.:  changed (nRow < 1)	to (nRow < 3)
   *  since the first row is the title row, which shouldn't count.  
   *  If nRow = 2, there is only 1 data row so it doesn't make
   *  sense to implement sorting either.
   **/
   // Should have at least 1 row
	if (nRow < 3)
   { 
      //alert("not sorting since < 2 data rows exist");
      return;
   }

	// Initialization of local variables
	actualNRow = 0;			// Number of actual data rows
	rNRowSpan = 0;			// Remaining rows in the row span
	rNColSpan = 0;			// Remaining cols in the col span
	nRowSpannedTitleCol = 0;	// Number of title cols from row span

	// Loop through rows
	for (var i=0; i<nRow; i++)
	{
		if (! separateTitle)
		{
			ctable = table;
		}
		else if (i == 0 && separateTitle && titleFound == false) 
		{
			ctable = ttable;
			nRow = ttable.rows.length + 1;
		}

		if (! doneftable && separateTitle && titleFound)
		{
			ctable = table;
			nRow = table.rows.length;
			i = 0;
			doneftable = true;
		}

		skipRow = false;
		// Skip if it's THEAD, TFOOT
		if (ctable.rows[i].parentNode != null)
		{
			parentNodeName = ctable.rows[i].parentNode.nodeName;
			parentNodeName.toUpperCase();
			if (parentNodeName == 'THEAD' ||
				parentNodeName == 'TFOOT')
			{
				skipRow = true;
			}
		}	
		nColSpan = 1, colPos = 0;
		// Loop through columns
		// Initializing
		for (var j=0; j<ctable.rows[i].cells.length; j++)
		{
			// Do this iff title has not been found
			if (titleFound == false)
			{
				if (ctable.rows[i].cells[j].rowSpan > 1)
				{
					if (ctable.rows[i].cells[j].colSpan < 2)
					{
						titleSpanCellArray[colPos] =
							ctable.rows[i].cells[j];
						titleRowArray[colPos] =
							ctable.rows[i];
						colTitleFilled[colPos] = true;
						nRowSpannedTitleCol++;
					}
					if (ctable.rows[i].cells[j].rowSpan - 1 
						> rNRowSpan)
					{
						rNRowSpan = 
							ctable.
							rows[i].cells[j].
							rowSpan - 1;

						if (ctable.rows[i].
							cells[j].colSpan > 1)
							rNColSpan = 
								rNRowSpan + 1;
					}
				}
			}
			if (ctable.rows[i].cells[j].colSpan > 1 &&
				rNColSpan == 0)
			{ 
				nColSpan = ctable.rows[i].cells[j].colSpan;
				colPos += nColSpan;
			}
			else
			{
				colPos++;
			}		
		}
					
		// Setting up the title cells
		if (titleFound == false && nColSpan == 1 && 
			rNRowSpan == 0 && rNColSpan == 0)
		{
			colSpanArray[i] = true;
			titleFound = true;

			// Using indivisual cell as an array element
			countCol = 0;
			for (var j=0;
				j<ctable.rows[i].cells.length
					+ nRowSpannedTitleCol; j++)
			{
				if (colTitleFilled[j] != true)
				{
					titleRowCellArray[j] =
						ctable.rows[i].cells[countCol];
					titleRowArray[j] =
						ctable.rows[i];
					countCol++;
				}
				else
				{
					titleRowCellArray[j] = 
						titleSpanCellArray[j];
					
				}
				titleInnerHTMLArray[j] =
					String(titleRowCellArray[j].innerHTML);
				titleSpanCountArray[j] = 
					titleRowCellArray[j].rowSpan;
			}
		}
		// Setting up the data rows
		else if (titleFound == true && nColSpan == 1 && 
			rNRowSpan == 0 && !skipRow)
		{
			for (var j=0; j<ctable.rows[i].cells.length; j++)
			{
				// Can't have row span in record rows ...
				if (ctable.rows[i].cells[j].rowSpan > 1) return;

				currentCell = ctable.rows[i].cells[j];
				cCellContent = String(currentCell.innerHTML);
				for (var k=0; k<cellPropArray.length; k++)
				{
					if (k == 0)
						cmd = "cCellSetting=" +
							"String(currentCell." +
							cellPropArray[k] + ");"
					else
						cmd = "cCellSetting+=" +
							"cellPropDelimiter+" +
							"String(currentCell." +
							cellPropArray[k] + ");"
					eval(cmd);
				}

				if (j == 0)
				{
					rowArray[actualNRow] = cCellContent;
					rowPropArray[actualNRow] = cCellSetting;
				}
				else
				{
					rowArray[actualNRow] += recDelimiter +
						cCellContent;
					rowPropArray[actualNRow] += 
						recDelimiter + cCellSetting;
				}
				if (j == ctable.rows[i].cells.length-1)
					rowArray[actualNRow] += recDelimiter +
						String(actualNRow);
			}
			// Inconsistent col lengh for data rows
			if (ctable.rows[i].cells.length > maxNCol)
         {
				return;
         }
			actualNRow++;
			colSpanArray[i] = false;
		}
		else if (nColSpan == 1 && rNRowSpan == 0 && 
			rNColSpan == 0 && titleFound == false && !skipRow)
		{
			colSpanArray[i] = false;
		}
		else
		{
			colSpanArray[i] = true;
		}
		
		// Counters for row/column spans
		if (rNRowSpan > 0) rNRowSpan--;
		if (rNColSpan > 0) rNColSpan--;
	}

	// If the row number is < 1, no need to do anything ...
	if (actualNRow < 1) return;

	// Sorting upon loading the page
	tIndex = findTableIndex(table.id);
	if (tIndex >= 0) tableLoaded = true;
	if (!tableLoaded)
	{
		tableIdArray[tableCount] = table.id;
		sortDirArray[tableCount] = false;
		tIndex = tableCount;
		tableCount++;
	}

	// Re-drawing the title row 
   if(!doInit_global)
   {
      redrawTitle(false);
   }

	// Sort if doSortUponLoad is set
	if (doSortUponLoad) 
	{
		sortTable(defaultSortColumn, table.id, 0);
		if (!tableLoaded) sortDirArray[tIndex] = true;
	}
   }// end of if(!disableSort)
}

//*****************************************************************************
// Function called to re-draw title row 
//*****************************************************************************
function redrawTitle(isSort)
{
	var currentRow, innerHTML, newInnerHTML, cellIndex;
	var reAnchor, reUpDown, reLabel, cellAlign, makeBold;
	var cellOnmouseover, cellOnmouseout;
 	var cellClass;
	var cmd;
	var cCellPropArray = new Array();
	var tIndex;

	cellAlign = "";
	makeBold = false;
	reAnchor = / *\<a[^\>]*\>(.*) *\<\/a\>/i;
	reUpDown = /\<font[^\>]* *id=.*updown.*[^\>]*\>.*\<\/font\>/i;
	reLabel = /\>([^\<]*)\</g;

	tIndex = findTableIndex(table.id);

	// Re-drawing the title row
	for (var j=0; j<maxNCol; j++)
	{
		currentRow = titleRowArray[j];
		innerHTML = String(titleInnerHTMLArray[j]);
		cellIndex = titleRowCellArray[j].cellIndex;

		// Skips the columns if set to ignore
		if (colsToIgnore != null)
			if (indexExist(colsToIgnoreArray,cellIndex)) continue;

		// Recording cell settings
	   for (var k=0; k<cellPropArray.length; k++)
		{
			cmd = "cCellPropArray[" + k + 
				"]=titleRowCellArray[j]." + 
				cellPropArray[k] + ";";
			eval(cmd);
		}
     
	  currentRow.deleteCell(cellIndex);
	  currentRow.insertCell(cellIndex);

		if (titleRowCellArray[j].tagName == "TH")
		{
			makeBold = true;
		}

		// Restoring cell settings
	   for (var k=0; k<cellPropArray.length; k++)
		{
			cmd = "currentRow.cells[cellIndex]." +
				cellPropArray[k] +
				"=cCellPropArray[" + k + "];";
			eval(cmd);
		}
     

		if (titleSpanCountArray[j] > 1)
			currentRow.cells[cellIndex].rowSpan = 
				titleSpanCountArray[j];
		newTitle = '';
		if (j == sortIndex && isSort)
		{
			newTitle = '<center><font id=updown color=' + 
				updownColor + '>&nbsp;';
			if (sortDirArray[tIndex])
				if (useImg)
					newTitle += '<img src="' +
						desChrFile + '" alt="' +
						desChr + '">';
				else
					newTitle += desChr;
			else
				if (useImg)
					newTitle += '<img src="' +
						ascChrFile + '" alt="' +
						ascChr + '">';
				else
					newTitle += ascChr;
			newTitle += '</font></center>';
		} 

		// Remove carriage return, linefeed, and tab
		innerHTML = innerHTML.replace(/\r|\n|\t/g, "");
      
      //the following bug fixed by Ha N.  
      //original code:  innerHTML.replace(reLabel, "<b>$1</b>")
		if (makeBold)
		{
			if (innerHTML.match(reLabel))
				innerHTML = 
					innerHTML.replace(reLabel, "><b>$1</b><");
			else
				innerHTML =
					innerHTML.replace(
						/(^.*$)/, "<b>$1</b>");
		}
      /**
      *  The following else statement added by Ha N.
      *  to resolve a possible bug.  After the first call to 
      *  redraw title, the <TH> tag is replaced by <TD> tag, making
      *  makeBold = false.  As a result, sorted column headers lose
      *  the <b> tag.  
      **/
      /************************************************/
      else
       {
          innerHTML = "<b>" + innerHTML + "</b>";

       }
      /***********************************************/

		innerHTML = innerHTML.replace(reUpDown, "");
		innerHTML = innerHTML.replace(reAnchor, "$1");
		newInnerHTML =
			'<a ' + linkEventString + j + ',' +
			tIndex + ',' + 1 + ',' + 1 + ')\', 1);">';
		if (useCustomTitleFont)
			newInnerHTML += '<font face="' + titleFont + '">' +
				+ innerHTML + '</font>';
		else
			newInnerHTML += innerHTML;
		newInnerHTML += '</a>' + newTitle;

      newInnerHTML = '<center>' + newInnerHTML +  '</center>';

		currentRow.cells[cellIndex].innerHTML = newInnerHTML;
		titleRowCellArray[j] = currentRow.cells[cellIndex];
	}
}


//*****************************************************************************
// Function called when user clicks on a title to sort
//
//*****************************************************************************
function sortTable(index,tobj,doInit,isIndex)
{
   doInit_global = doInit;
	
   var obj, tIndex;
	if (isIndex != null)
		if (isIndex)
		{
			tIndex = tobj;
			// Look up the table object
			obj = tableIdArray[tobj];
		}	
		else
		{
			tIndex = findTableIndex(tobj.id);
			obj = tobj;
		}

   if(firstTableInit && doRowCheck)
   {
      removeRowDividers();
      firstTableInit = false;
      initTable(obj, statusString, sortProgressString, datePattern, null, colsToIgnore, doRowCheck);
   }

   // Re-initializing the table object if more than 1 table exist in a single servlet
	if (tableCount > 1) 
	{     
			initTable(obj, statusString, sortProgressString, datePattern, null, colsToIgnore, doRowCheck);
	}
	
   // Local variables
	var rowContent, rowProp, cellProp;
	var rowCount;
	var rowIndex;
	var cmd;
   var progressCount = 0;
   var percentComplete = 40;
   var updateIncrement = parseInt(nRow/4);

   //don't bother with the sorting status update if the table
   //contains less than 40 rows 
   if(updateIncrement < 10)
   {
      updateIncrement = -1;
   }
   else
   {
      updateProgressMessage(0);
   }

	
	// Can't sort past the max allowed column size
	if (index < 0 || index >= maxNCol) return;
	
	// Assignment of sort index
	sortIndex = index;

   sort_By_Data_Type();

   if(updateIncrement != -1)
   {
       updateProgressMessage(20);
   }


	// Re-drawing the title row
	if (doInit || doSortUponLoad) redrawTitle(true);

	rowCount = 0;
   // Re-drawing the sorted table
	for (var i=0; i<nRow; i++)
   {
		if (! colSpanArray[i])
		{
			for (var j=0; j<maxNCol; j++)
			{
				// Skips the columns if set to ignore
				if (colsToIgnore != null)
					if (indexExist(colsToIgnoreArray,j)) 
						continue;
				rowContent = rowArray[rowCount].
					split(recDelimiter);
				rowIndex = rowContent[maxNCol];
				rowProp = rowPropArray[rowIndex].
					split(recDelimiter);
				cellProp = rowProp[j].split(cellPropDelimiter);

				// Restoring cell properties
				for (var k=0; k<cellPropArray.length; k++)
				{
					cmd = "table.rows[i].cells[j]." +
						cellPropArray[k] +
						"=cellProp[" + k + "];";
					eval(cmd);
               
				}
            /************************************
            *  THE FOLLOWING LINE WAS ADDED BY HA N.
            *  Although cellProp[3], which is noWrap, is = false before and after eval(cmd),
            *  table.rows[].cells[].noWrap begins with false before eval() and ends with true
            *  after the command.  Since it is desired for noWrap = true, I manually set it here.
            ************************************/
            table.rows[i].cells[j].noWrap = false;
				table.rows[i].cells[j].innerHTML = rowContent[j];
			}
			rowCount++;
         progressCount++;

         if(progressCount == updateIncrement)
         {
            updateProgressMessage(percentComplete);
            progressCount = 0;
            percentComplete = percentComplete + 20;
         }
		}
	}

	// Switching btw descending/ascending sort
	if (doInit)
	{
		if (sortDirArray[tIndex])
			sortDirArray[tIndex] = false;
		else
			sortDirArray[tIndex] = true;
	}
	setCursorDefault();
}

//*****************************************************************************
// Function to be used for Array sorting
//*****************************************************************************
function compare(a, b)
{
	// Getting the table index
	var tIndex;
	tIndex = findTableIndex(table.id);
 
	// Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);
	
	// Needed in case the data conversion is necessary
	var aToBeCompared, bToBeCompared;

	// Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");
   
   if (! isNaN(aRowContent[sortIndex]) &&
		! isNaN(bRowContent[sortIndex]))
	{
		aToBeCompared = parseFloat(aRowContent[sortIndex], 10);
		bToBeCompared = parseFloat(bRowContent[sortIndex], 10);
	}
	else
	{
		aToBeCompared = aRowContent[sortIndex];
		bToBeCompared = bRowContent[sortIndex];
	}

	if (aToBeCompared < bToBeCompared)
		if (!sortDirArray[tIndex])
		{
			return -1;
		}
		else
		{
			return 1;
		}
	if (aToBeCompared > bToBeCompared)
		if (!sortDirArray[tIndex])
		{
			return 1;
		}
		else
		{
			return -1;
		}
	return 0;
}

//*****************************************************************************
// Functions to set the cursor
//*****************************************************************************
function setCursor(obj)
{
	var rowText, reRowText;

	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	// Show hint text at the browser status bar
	rowText = String(obj.innerHTML);

	// Remove up/down 
	rowText = rowText.replace(/\<font id\=updown.*\<\/font\>/ig, "");
	// Remove HTML tags and &nbsp;
	rowText = rowText.replace(reRowText, "");
	// Remove carriage return, linefeed, and tab
	rowText = rowText.replace(/\r|\n|\t/g, "");
	
	// Setting window's status bar - modified by Ha N.
	window.status = statusString + " " + String(rowText);

	// Setting title
	obj.title = statusString + " " + String(rowText);

	// Change the mouse cursor to pointer or hand
	if (isIE)
		obj.style.cursor = "hand";
	else
		obj.style.cursor = "pointer";
}

//Routine modified by Ha N. to increase performance
function setCursorWait(obj)
{  
   obj.style.cursor = "wait";
   if(!(isIE && document.all))
   {
      document.body.style.cursor = 'wait';
   }
}


//Routine modified by Ha N. to increase performance
function setCursorDefault()
{
   //if (isIE && document.all)
		//obj.style.cursor = '';
	//else
		document.body.style.cursor = '';  
      window.status = '';
}

//*****************************************************************************
// Function to set the title color
//*****************************************************************************
function setColor(obj,mode)
{
	if (mode == "selected")
	{
		// Remember the original color
		if (obj.style.color != selectedColor) 
			defaultColor = obj.style.color;
		obj.style.color = selectedColor;

		if (highlightSelCell)
      {
			obj.bgColor = cellHighlightColor;
      }
	}
	else
	{	
		// Restoring original color and re-setting the status bar
		obj.style.color = defaultColor;
		if (highlightSelCell) obj.bgColor = defaultColor;
		window.status = '';
	}
}

//*****************************************************************************
// Function to check browser type/version
//*****************************************************************************
function checkBrowser()
{
   //added by Ha N. to detect NS4
   if(document.layers)
   {
      return false;
   }

	if (navigator.appName == "Microsoft Internet Explorer"
		&& parseInt(navigator.appVersion) >= 4)
	{
		isIE = true;
		return true;
	}
   // this else if statement doesn't detect NS4
	// For some reason, appVersion returns 5 for Netscape 6.2 ...
	else if (navigator.appName == "Netscape"
		&& navigator.appVersion.indexOf("5.") >= 0)
	{
		isIE = false;
		return true;
	}
	else
		return false;
}

//*****************************************************************************
// Function to check whether index exists in array 
//*****************************************************************************
function indexExist(x, index)
{
	var found = false;
	for (var i=0; i<x.length; i++)
	{
		if (index == x[i])
		{
			found = true;
			break;
		}
	}
	return found;
}

//*****************************************************************************
// Function to return table index given table id 
//*****************************************************************************
function findTableIndex(id)
{
	for (var i=0; i<tableCount; i++)
		if (tableIdArray[i] == id)
			return(i);
	return(-1);
}



//====================================================
//====================================================
//Functions added by Ha N.
//====================================================
//====================================================

/**
*  Many NVIP tables contain values that cannot be correctly sorted alphabetically or 
*  numerically (i.e. time stamps).  This function determines the data type of the column
*  being sorted and calls an appropriate sorting algorithm.
*
*  TABLE NAMING CONVENTIONS USED BY HA N.
*  ========================================
*  In order simplify the process of determining the sorting needs of each NVIP table, 
*  the table id's are created by concatenating a string of data types for each column, 
*  seperated by the '*' character.
*  
*  THE POSSIBLE DATA TYPES ARE:
*  ============================
*     id = IP address or String (ie application name, buffer name, etc)
*     time = short date (ie MM/dd/yyyy) or String ="Hour #" or String ="Minute #"
*     num = a number which may contain commas
*     time2 = a time string resembling HH:mm, HH:mm:ss, or HH:mm:ss:SS
*     string = a normal string
*     ip = ip address
*     percent = percentage value
*     mibtime = "# day(s) #:#:#:#" or just "#:#:#:#"
*     lparNum = "#.#"
* 
**/
function sort_By_Data_Type()
{     
   var dataTypeTable = table.id.split("*");
   var dataType = dataTypeTable[sortIndex];
   
   if(dataType == "id")
   {
      rowArray.sort(compareID);
   }
   else if(dataType == "time")
   {
      rowArray.sort(compareTime);
   }
   else if(dataType == "time2")
   {
      rowArray.sort(compareTimeOfDay);
   }
   else if(dataType == "num")
   {
      rowArray.sort(compareNumWithCommas);
   }
   else if(dataType == "ip")
   {
      rowArray.sort(compareIPAddress);
   }
   else if(dataType == "percent")
   {
      rowArray.sort(comparePercent);
   }
   else if(dataType == "timeStamp")
   {
      rowArray.sort(compareTimeStamp);
   }        
   else if(dataType == "mibTime")
   {
      rowArray.sort(compareMibTime);
   }
   else if(dataType == "lparNum")
   {
      rowArray.sort(compareLparNum);
   }
   //dataType == "string"
   else
   {
      rowArray.sort(compare);
   }
        
}

//########################################################################
// CUSTOMIZED COMPARE FUNCTIONS FOR SPECIFIC NVIP FORMATTED DATA
//########################################################################

/**
*  This function compares 2 time stamps of the format "# secs # ms".  If the first time stamp
*  is greater than the second, 1 is returned.  If it is less than the second, -1 is returned.
*  If the 2 timestamps are equal, 0 is returned.
**/
function compareTimeStamp(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var time1 = aRowContent[sortIndex];
   var time2 = bRowContent[sortIndex];
   var sec1, sec2, milliSec1, milliSec2;
   var time1_tokens =  time1.split(/\s+/);
   var time2_tokens =  time2.split(/\s+/);

   //the first token will always contain a number representing the seconds
   sec1 = time1_tokens[0];
   sec2 = time2_tokens[0];


   //The word 'secs' actually comes from a resource bundle.  Thus, there is the 
   //possibility that the resource bundle returns several words.  Therefore,
   //we must find which token represents the milliSecs.
   
   var tempToken;
   var tempInt;
   var counter = 3;

   if(time1_tokens.length == 4)
   {
      milliSec1 = time1_tokens[2];
   }
   else
   {
      tempToken = time1_tokens[counter];
      tempInt = parseInt(tempToken);
       
      while(isNaN(tempInt))
      {
         counter++;
         tempToken = time1_tokens[counter];
         tempInt = parseInt(tempToken);
      }

      //important to take token not tempInt because if the milliSec contains commas,
      //tempInt will return the numbers that appear before the first comma
      milliSec1 = time1_tokens[counter];
      
   }
   //reset the counter
   counter = 3;

   if(time2_tokens.length == 4)
   {
      milliSec2 = time2_tokens[2];
   }
   else
   {
      tempToken = time2_tokens[counter];
      tempInt = parseInt(tempToken);
       
      while(isNaN(tempInt))
      {
         counter++;
         tempToken = time2_tokens[counter];
         tempInt = parseInt(tempToken);
      }

      //important to take token not tempInt because if the milliSec contains commas,
      //tempInt will return the numbers that appear before the first comma
      milliSec2 = time2_tokens[counter];
   }

   var sec1_int, milliSec1_int, sec2_int, milliSec2_int;

   sec1_int = parseInt(removeCommasFromNum(sec1));
   milliSec1_int = parseInt(removeCommasFromNum(milliSec1));
   sec2_int = parseInt(removeCommasFromNum(sec2));
   milliSec2_int = parseInt(removeCommasFromNum(milliSec2));


   //now do the actual compare
   var compareResult;
   compareResult = compareProcessedValues(sec1_int, sec2_int);

   //seconds are equal so compare milliSecs
   if (compareResult == 0)
	{	
      compareResult = compareProcessedValues(milliSec1_int, milliSec2_int);
   }
   return compareResult;
 }


function compareNumWithCommas(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aTemp, bTemp;
   aTemp = removeCommasFromNum(aRowContent[sortIndex]);
   bTemp = removeCommasFromNum(bRowContent[sortIndex]);

   return compareProcessedValues(aTemp, bTemp);
}



function comparePercent(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aPercent = extractNumFromString(aRowContent[sortIndex]);
   var bPercent = extractNumFromString(bRowContent[sortIndex]);

   return compareProcessedValues(aPercent, bPercent);
}



function compareIPAddress(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aIndex, bIndex;
   var aAddressField, bAddressField;
   var aAddress, bAddress;
   var compareResult;

   aAddress = aRowContent[sortIndex];
   bAddress = bRowContent[sortIndex];
    

   for(var addressSegment = 0; addressSegment < 4; addressSegment++)
   {

      if(addressSegment != 3)
      {
         aIndex = aAddress.indexOf(".");
         bIndex = bAddress.indexOf(".");

         aAddressField = aAddress.substring(0, aIndex);
         bAddressField = bAddress.substring(0, bIndex);
      }
      else
      {
         aAddressField = aAddress;
         bAddressField = bAddress;
      }
      
      
      compareResult = compareProcessedValues(aAddressField, bAddressField);
      
      if(compareResult != 0)
      {
         return compareResult;
      }
      else
      {
         aAddress = aAddress.substring(aIndex + 1);
         bAddress = bAddress.substring(bIndex + 1);
      }
   }//end of for loop

   return 0;
}



/**
*  This function is used primarily by historical reports that have the "Weekly" or "Hourly"
*  report option.  Depending on which option is chosen, the data columns will either 
*  be short dates (i.e. MM/dd/yyyy) or strings indicating the hour of day (i.e. "Hour 0").
**/
function compareTime(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aTime, bTime;
   var monthIndex, dayIndex, yearIndex, aMonth, aDay, aYear, bMonth, bDay, bYear, compareResult;

   aTime = aRowContent[sortIndex];
   bTime = bRowContent[sortIndex];

   //The 2 times are not dates; they are strings representing either the hour of day
   //of minute of hour.  For this case, extract either the minute or hour and compare the values.
   if(datePattern == null)
   {
      aTime = extractNumFromString(aTime);
      bTime = extractNumFromString(bTime);
      return compareProcessedValues(aTime, bTime);
   }
   //The args are dates except the date given doesn't follow the date pattern specified so don't 
   //sort at all!
   else if((aTime.length != datePattern.length) || (bTime.length != datePattern.length))
   {
      return 0;
   }

   monthIndex = datePattern.indexOf("MM");
   dayIndex = datePattern.indexOf("dd");
   yearIndex = datePattern.indexOf("yyyy");

   aMonth = aTime.substring(monthIndex, monthIndex + 2);
   aDay = aTime.substring(dayIndex, dayIndex + 2);
   aYear = aTime.substring(yearIndex, yearIndex + 4);
   bMonth = bTime.substring(monthIndex, monthIndex + 2);
   bDay = bTime.substring(dayIndex, dayIndex + 2);
   bYear = bTime.substring(yearIndex, yearIndex + 4);


   compareResult = compareProcessedValues(aYear, bYear);
   
   //the 2 dates have the same year value
   if(compareResult == 0)
   {
      compareResult = compareProcessedValues(aMonth, bMonth);

      //the 2 dates have the same month value
      if(compareResult == 0)
      {
         compareResult = compareProcessedValues(aDay, bDay);
      }
   }

   return compareResult;
}



function compareTimeOfDay(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aTime, bTime;
   var hourIndex, minuteIndex, secIndex, milliSecIndex, aHour, aMinute, aSec, aMilliSec, bHour, bMinute, bSec, bMilliSec, compareResult;
   var argLength;
   var timePattern;

   aTime = aRowContent[sortIndex];
   bTime = bRowContent[sortIndex];
   argLength = aTime.length;


   if(argLength == shortTimePattern.length)
   {
      timePattern = shortTimePattern;
   }
   else if(argLength == mediumTimePattern.length)
   {
      timePattern = mediumTimePattern;
   }
   else if(argLength == longTimePattern.length)
   {
      timePattern = longTimePattern;
   }
   else
   {
      //args given don't match any of our time patterns so don't sort at all
      return 0;
   }

   hourIndex = timePattern.indexOf("HH");
   minuteIndex = timePattern.indexOf("mm");
   secIndex = timePattern.indexOf("ss");
   milliSecIndex = timePattern.indexOf("SS");


   aHour = aTime.substring(hourIndex, hourIndex + 2);
   aMinute = aTime.substring(minuteIndex, minuteIndex + 2);
   aSec = "00";
   aMilliSec = "00";
   
   bHour = bTime.substring(hourIndex, hourIndex + 2);
   bMinute = bTime.substring(minuteIndex, minuteIndex + 2);
   bSec = "00";
   bMilliSec = "00";
   
   if(timePattern != shortTimePattern)
   {
      aSec = aTime.substring(secIndex, secIndex + 2);
      bSec = bTime.substring(secIndex, secIndex + 2);
   }
   if(timePattern != mediumTimePattern)
   {
      aMilliSec = aTime.substring(milliSecIndex, milliSecIndex + 2);
      bMilliSec = bTime.substring(milliSecIndex, milliSecIndex + 2);
   }

   compareResult = compareProcessedValues(aHour, bHour);
   
   //the 2 dates have the same hour value
   if(compareResult == 0)
   {
      compareResult = compareProcessedValues(aMinute, bMinute);

      //the 2 dates have the same minute value
      if(compareResult == 0)
      {
         compareResult = compareProcessedValues(aSec, bSec);

         //the 2 dates have the same second value
         if(compareResult == 0)
         {
            compareResult = compareProcessedValues(aMilliSec, bMilliSec);
         }

      }
   }

   return compareResult;

}


/**
*  Many SNMP MIB tables have columns with timestamps of the form "# day(s) #:#:#.#" 
*  or just "#:#:#.#".  The first "#" could either be 1 or 2 digits long.  The remaining
* "#'s" are exactly 2 digits long.    
**/
function compareMibTime(a, b)
{
   // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aStamp, bStamp;
   var aDays, bDays, compareResults;
   var aTime, bTime;


   aStamp = aRowContent[sortIndex].split(" ");
   bStamp = bRowContent[sortIndex].split(" ");
   
   aDays = 0;
   bDays = 0;
   aTime = aStamp[0];
   bTime = bStamp[0];

   if(aStamp.length > 1)
   {
      aDays = aStamp[0];
      aTime = aStamp[2];
   }
   if(bStamp.length > 1)
   { 
      bDays = bStamp[0];
      bTime = bStamp[2];
   }

   //If time stamp begins with a single digit in the first field,
   //pad it with a '0' so that we can use a string comparison on the 
   //times.
   if(aTime.length == 10)
   {
      aTime = "0" + aTime;
   }
   if(bTime.length == 10)
   {
      bTime = "0" + bTime;
   }

   compareResults = compareProcessedValues(aDays, bDays);
   
   if(compareResults != 0)
   {
      return compareResults;
   }
   else 
   {
      return compareProcessedValues(aTime, bTime);
   }
}


/**
*  This function sorts the "LPAR Number" column of the Osa Performance 
*  PCI Bus/Processor Load table. Values of this type are of the form
*  "#.#". 
**/
function compareLparNum(a, b)
{
    // Getting the element array for inputs (a,b)
	var aRowContent = a.split(recDelimiter);
	var bRowContent = b.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");
	bRowContent[sortIndex] = bRowContent[sortIndex].replace(reRowText, "");

   var aValue, bValue;
   var compareResults, length, aTemp, bTemp;

   aValue = aRowContent[sortIndex].split(".");
   bValue = bRowContent[sortIndex].split(".");

   length = aValue.length;
   if(length > bValue.length)
   {
      length = bValue.length;
   }

   for(var i = 0; i < length; i++)
   {
      aTemp = aValue[i];
      bTemp = bValue[i];
      compareResults = compareProcessedValues(aTemp, bTemp);
      if(compareResults != 0)
      {
         return compareResults;
      }
   }
   return compareResults;
}


/**
*  Certain tables contain an ID column (ie Application Name, Month, IP address) for which
*  the data in the remaining columns pertain.  The function checks to see if the ID values
*  are strings or IP addresses and compares them accordingly.  
**/


function compareID(a, b)
{       
   if(isIPAddressColumn(a))
   {                          
      return compareIPAddress(a, b);
   }
   else
   {
      return compare(a, b);
   }
}


//########################################################################
// FORMATTING HELPER FUNCTIONS USED IN CUSTOMIZED COMPARE FUNCTIONS
//########################################################################


/**
*  This function takes a string representing a number that contains
*  commas.  The function removes the commas from the string and returns
*  the string after calling parseInt.  If removing the commas from the 
*  string argument still results in parseInt(processed_arg) = NaN, the 
*  original string is returned.  
**/
function removeCommasFromNum(numString)
{
   var tempNum = numString;

   if(isNaN(tempNum))
   {
      tempNum = numString;
      
      var commaIndex = tempNum.indexOf(",");
      while(commaIndex != -1)
      {
         tempNum = tempNum.substring(0, commaIndex) + tempNum.substring(commaIndex + 1);
         commaIndex = tempNum.indexOf(",");
      }
      tempNum = parseInt(tempNum);
   }


   //if we still don't have a number, return the original argument
   if(isNaN(tempNum))
   {
      tempNum = numString;

   }
   return tempNum;
}



function extractNumFromString(stringWithNum)
{
   var tempValue;
   var beginIndex, endIndex;
   var tempString;

   beginIndex = 0;
   endIndex = 1;
   tempValue = stringWithNum.substring(beginIndex, endIndex);
   
   //find where the number in the string begins
   while(isNaN(tempValue) && endIndex < stringWithNum.length)
   {
      beginIndex++;
      endIndex++;
      tempValue = stringWithNum.substring(beginIndex, endIndex);
   }
  
   //find where the number in the string ends
   while(endIndex < stringWithNum.length)
   {
      endIndex++;
      if(endIndex == stringWithNum.length)
      {
        tempString =  stringWithNum.substring(beginIndex);
      }
      else
      {
         tempString =  stringWithNum.substring(beginIndex, endIndex);
      }
      
      if(isNaN(tempString))
      {
         break;
      }
      else
      {
         tempValue = tempString;
      }
      
   }
   return tempValue;
 }

function compareProcessedValues(a, b)
{
   var aToBeCompared, bToBeCompared; 
	var tIndex;
	
   tIndex = findTableIndex(table.id);

   if (! isNaN(a))
      aToBeCompared = parseFloat(a, 10);
   else
      aToBeCompared = a;

   if (! isNaN(b))
      bToBeCompared = parseFloat(b, 10);
   else
      bToBeCompared = b;

   if (aToBeCompared < bToBeCompared)
   {        
  //   alert("inside compareProcessValues:  " + aToBeCompared + " is less than " +  bToBeCompared);
      if (!sortDirArray[tIndex])
      {
         return -1;
      }
      else
      {
         return 1;
      }
   }
      
   if (aToBeCompared > bToBeCompared)
   {

     // alert("inside compareProcessValues:  " + aToBeCompared + " is greater than " +  bToBeCompared);
      if (!sortDirArray[tIndex])
      {
         return 1;
      }
      else
      {
         return -1;
      }
   }

  //alert("inside compareProcessValues:  " + aToBeCompared + " is equal to " +  bToBeCompared);
   return 0;
}




function isIPAddressColumn(a)
{
   // Getting the element array for input a
	var aRowContent = a.split(recDelimiter);

   // Remove HTML tags 
	reRowText = /(\< *[^\>]*\>|\&nbsp\;)/g;
	aRowContent[sortIndex] = aRowContent[sortIndex].replace(reRowText, "");

   var startIndex, endIndex;
   var aID, tempID, tempSubstring;
   var isIPAddress;

   aID = aRowContent[sortIndex];

   //check to see if one of the ID's is an IP address
   tempID = aID;
   startIndex = 0;
   endIndex = 0;
   
   for(var i = 0; i < 3; i++)
   {
      
      if(i == 0)
      {
         endIndex = tempID.indexOf(".");
      }
      else if(i == 1)
      {
            startIndex = tempID.indexOf(".");
            endIndex = tempID.substring(startIndex + 1).indexOf(".");
      }
      else
      {
         startIndex = tempID.indexOf(".");
      }
      
      if( (startIndex == -1) || (endIndex == -1))
      {
         return false;
      }
      else
      {
         if(i == 0)
         {
            tempSubstring = tempID.substring(startIndex, endIndex);
         }
         else if(i == 1)
         {
            tempSubstring = tempID.substring(startIndex + 1, endIndex);
         }
         else
         {
            tempSubstring = tempID.substring(startIndex + 1);
         }
         
         if(isNaN(tempSubstring))
         {             
            return false;
         }
      }

      tempID = tempID.substring(endIndex);
   }//end of for loop
      return true;
}


/**
*  Some NVIP tables contain rows that act purely as dividers.  Each data 
*  cell of these divider rows contain the "*" symbol.  Prior to the first 
*  call to sortTable(), this function is called to remove all row 
*  dividers from the table.
**/
function removeRowDividers()
{
   var numOfRows, numOfColumns;
   var currentRow;
   var removeRow;

   numOfRows = table.rows.length;

   for(var i = 0; i < numOfRows; i++)
   {
      currentRow = table.rows[i];
      numOfColumns = currentRow.cells.length;
      //alert("there are this many columns :  " + numOfColumns);
      removeRow = true;
       
      for(var x = 0; x < numOfColumns; x++)                        
      {  
         if(currentRow.cells[x].innerText.replace(/\s*([^\s]*)\s*/g, "$1") != "*")
         {
            removeRow = false;
            break;
         }
         
      }
       
      
      if(removeRow)
      {
         table.deleteRow(i);
         i--;
         numOfRows--;
      }
      
   }
}

/**
*  This function updates the progress message displayed on the user's browser while a
*  table is being sorted.  The function is called for every 20% of the rows that is sorted.
**/
function updateProgressMessage(percentComplete)
{
     /*
   var percentComplete = parseInt(rowsCompleted/nRow * 100);

   alert(percentComplete + "% of sorting completed.");
   window.status = percentComplete + sortProgressString;
   */
   window.status = percentComplete + sortProgressString;
}
